/*
 * Copyright (C) 2001 Networks Associates Technology, Inc.
 * All rights reserved.
 *
 * $Id: APIThreadSample.c,v 1.3 2003/09/05 17:13:20 ishabtay Exp $
 * 
 * This is a sample program to demonstrate the PGP E-Business multi-threaded API
 *
 */

#include <stdio.h>
#include <stdlib.h>

#ifndef PGP_WIN32
#include <pthread.h>
#endif

#include "PGPeBiz.h"    /* API header, must be included in all API programs */

#define STR_LEN 1024	/* generic length for tmp strings */

#define NUM_THREADS 3	/* number of threads to spawn */

/* this structure is useful for passing PGPeBiz() parameters */
typedef struct _threadCallStruct {
    PGPeBizContextRef context;	        /* eBiz context */
    PGPByte *stdOut, *stdIn, *stdErr;	/* I/O buffers */
    PGPUInt32 outLen, inLen, errLen;	/* buffer lengths */
    char commandString[STR_LEN + 1];	/* the command string */
	int status;	                        /* status */
} threadCallStruct;

/* Win32 specific stuff */
#ifdef PGP_WIN32
#define snprintf _snprintf
#include <windows.h>
#include <process.h>
#endif /* win32 */

void spawnEbiz(threadCallStruct * args)

 {
    int status;

    status = PGPeBiz(args->context, args->commandString, args->stdIn,
		     args->inLen, &(args->stdOut), &(args->outLen),
		     &(args->stdErr), &(args->errLen));
	
	args->status = status;

    return ;
} /* spawnEbiz */

int main(int argc, char *argv[])
 {
    char errorString[STR_LEN + 1];		/* string for error string */
    PGPeBizContextRef context;	        /* eBiz context */
    threadCallStruct args[NUM_THREADS];	/* array of PGPeBiz() args */
#ifdef PGP_WIN32
	HANDLE threads[NUM_THREADS];
	unsigned tid;
#else
    pthread_t threads[NUM_THREADS];		/* array of thread ids */
#endif
    int initStatus;                    
    int index;
    int anykey;

    /* This function must be called before any other API calls.
     * You must save the "context" for future API calls.
     */
    initStatus = PGPeBizInit(&context);
    if (initStatus != 0) {
		fprintf(stderr, "Could not initialize context\n");
		exit(1);
    }

    /* we'll go ahead and load up the args for the threads we will spawn */
    snprintf(args[0].commandString, STR_LEN, "--key-list");
    args[0].context = context;	/* from PGPeBizInit() call */
    args[0].stdIn = NULL;	/* we have no input data */

    /* second thread */
    snprintf(args[1].commandString, STR_LEN,
	     "--encrypt --conventional --passphrase \"test this\" --overwrite "
	     "--output test.txt.pgp");

    /* allocate the stdin buffer, we will need to free this when done */
    if ((args[1].stdIn = (PGPByte *) malloc(STR_LEN + 1)) == NULL) {
		fprintf(stderr,
		"I could not malloc the input buffer....  exiting....\n");
		exit(1);
    }

    /* fill stdin buffer with data to be encrypted */
    args[1].inLen = snprintf(args[1].stdIn, STR_LEN,
			     "This is a test\nThis is only a test.\nThank you "
			     "for your support.\n");
    args[1].context = context;	/* from PGPeBizInit() call */

    /* third thread */
    snprintf(args[2].commandString, STR_LEN,
	     "--decrypt test.txt.pgp --passphrase \"test this\" --output -");
    args[2].context = context;
    args[2].stdIn = NULL;

    printf("Spawning threads....\n\n\n");

    /* spawn the threads */
    for (index = 0; index < NUM_THREADS; index++) {
		printf("...%d...\n", index);
#ifdef PGP_WIN32
		threads[index] = (HANDLE) _beginthreadex(NULL, 0, (void *) spawnEbiz,
				(void *) &(args[index]), 0, &tid);
#else
	    pthread_create(&(threads[index]), NULL, (void *) spawnEbiz,
				   (void *) &(args[index]));
#endif
	} /* spawn threads */

    printf("Waiting for threads....\n\n\n");

    /* wait for threads to finish before printing results */
    for (index = 0; index < NUM_THREADS; index++) {

#ifdef PGP_WIN32
		WaitForSingleObject(threads[index], INFINITE);
#else
		pthread_join(threads[index], NULL);
#endif

		printf("Hit <RETURN> to see output from thread %d\n", index);
		anykey = getchar();

		printf("********  BEGIN output thread %d\n", index);
		printf("Command is [%s]\n", args[index].commandString);
		if (args[index].outLen != 0)
		  printf("stdout = [%.*s] = %d bytes\n", (int) args[index].outLen,
			 args[index].stdOut, args[index].outLen);
		else
		  printf("stdout is empty\n");
		if (args[index].errLen != 0)
		  printf("stderr = [%.*s] = %d bytes\n", (int) args[index].errLen,
			 args[index].stdErr, args[index].errLen);
		else
		  printf("stderr is empty\n");
		printf("PGPeBiz() returned a code of %d which is %s\n",
		       args[index].status, PGPeBizErrorString(context, args[index].status,
						 errorString, 1000));
		printf("XXXXXXXX END output thread %d\n\n", index);

    } /* wait for threads to finish */

    /* When you are finished with API allocated buffers they must be freed */
    for (index = 0; index < NUM_THREADS; index++) {
		PGPeBizFree(context, args[index].stdOut);
		PGPeBizFree(context, args[index].stdErr);
    }

    /* free the stdin buffer we allocated since we are done with it */
    free(args[1].stdIn);

    /* This should be called last to free up the eBiz context.
     * If you want to make any more API calls after freeing the
     * context, you must call PGPeBizInit() to create a new context
     * first.
     */
    PGPeBizCleanUp(context);

    return (0);
} /* main */
